/***
* Name: SamambaiaBasinHydroABM
* Author: Pedro Phelipe - new.petter91@gmail.com
* Description: An ABM of an Basin (Samambaia Basin) built to explore scenarios of water management and access its the impacts
* Tags: Basin, Water Management, Agent-Based Model
***/

model SamambaiaBasinHydroABM

/* Insert your model definition here */

global {
	int nb_farmerA <- 2; //number of farmer A
	int nb_farmerB <- 5; //number of farmer B
	int nb_farmerC <- 3; //number of farmer C
	float step <- 1 #day; //time of each step
	float totalcrop1; //total area planted of crop 1
	float totalcrop2;
	float totalcrop3;
	float totalcrop4;
	float totalcrop5;
	float totalcrop6;
	float totalcrop7;
	float totalcrop8;
	float totalcrop9;
	float totalkgcrop1; //total kg produced of crop 1
	float totalkgcrop2;
	float totalkgcrop3;
	float totalkgcrop4;
	float totalkgcrop5;
	float totalkgcrop6;
	float totalkgcrop7;
	float totalkgcrop8;
	float totalkgcrop9;
	float totalflowdemand;
	float global_dry_area;
	float ppcost; //pumping cost 0.0002 TODO
	float PPU; //unitary cost of water
	float inst_wdem;
	float count_year; //added to count years and will be used inside matrix
	float cyear; // help to count years
	float last_year; //is used to take weather data from last year
	float dayly_withdraw; //water available daily
//	float avlb_water;
	float basin_rain;
	//saving atributes
	float total_av_water; //total water available
	float total_dem_water; // total water demand
	float total_cash; //total cash inside basin
	file cropland <- file("../includes/pivos_bacia_2016.shp"); //shapefiles of cropfields
	file river <- file("../includes/rios_bacia.shp");
	file basin <- file("../includes/bacia_samambaia.shp");
	file rato <- file("../includes/cor_rato.shp");
	file sam_norte <- file("../includes/sam_norte.shp");
	file sam_sul <- file("../includes/sam_sul.shp");
	file weatherdata <- csv_file("../includes/weather.csv"); //csv file with weather data
	file prices <- csv_file("../includes/prices.csv");
	file fsamn <- csv_file("../includes/fsamn.csv");
	file fsams <- csv_file("../includes/fsams.csv");
	file frato <- csv_file("../includes/frato.csv");
	geometry shape <- envelope(basin); //delimitating simulation area

	init { //creating agents
		create cropfield from: cropland; //creating cropfield from shapefile
		create weather number: 1; //creating 1 weather
		create calendar number: 1; // creating 1 calendar
		create market number: 1; // creating 1 market
		create watershed from: basin; //creating watersehd from shapefile
		create visual_river from: river; //creating river from shapefile
		create cor_rat from: rato; // creating cor_rat from shapefile
		create sam_north from: sam_norte; //creating sam_north from shapefile
		create sam_south from: sam_sul; //creating sam_south from shapefile
		create farmerC number: nb_farmerC { //creating farmer C arround cor_rat
			location <- any_location_in(4000 around cropfield(246));
		}

		create farmerB number: nb_farmerB { // creating farmer B around sam_north
			location <- any_location_in(4000 around cropfield(9));
		}

		create farmerA number: nb_farmerA { //creating farmer A around sam_south
			location <- any_location_in(4000 around cropfield(118));
		}

		count_year <- 0;
		cyear <- current_date.year;
	}

	reflex { // reflex that counts years (its used to select the row of the matrix)
		if (current_date.year != cyear) {
			count_year <- count_year + 1.0;
			cyear <- current_date.year;
		}

		if (count_year = 11.0) {
			count_year <- 1.0;
		}
		last_year <- count_year -1;
		if(last_year = 0){
			last_year <- 10;
		}
		if (count_year < 3){
			PPU <- 0.000015;
		}
		if (count_year = 3){
			PPU <- 0.000020;
		}if (count_year = 4){
			PPU <- 0.000022;
		}if (count_year >= 5){
			PPU <- 0.000025;
		}
		
		ppcost <- 0.0002; // + (PPU*0.1); //multiplied by 0.1 due to agricultural usage and class 2 river (on legislation) TODO

	}

	reflex { //calc some indicators
		inst_wdem <- sum(farmerA collect each.day_wdem) + sum(farmerB collect each.day_wdem) + sum(farmerC collect each.day_wdem);
		dayly_withdraw <- sum(cor_rat collect each.withdraw) + sum(sam_north collect each.withdraw) + sum(sam_south collect each.withdraw);
		basin_rain <- sum(weather collect each.rain);
		total_av_water <- total_av_water + sum((cor_rat, samn_north, sam_south) collect each.withdraw);
		total_dem_water <- sum((farmerA, farmerB, farmerC) collect each.total_water_demand);
		total_cash <- total_cash + sum((farmerA, farmerB, farmerC) collect each.cash);
		global_dry_area <- global_dry_area + sum(cropfield collect each.dry_area);
	}

	reflex end_simulation when: cycle = 3600 { //pause simulation after 10 years and save results
		save [total_av_water, total_dem_water, total_cash] to: "../results/total_result.csv" type: "csv" rewrite: false;
		ask farmerA {
			save [name, my_area, total_water_demand, cash] to: "../results/each_result.csv" type: "csv" rewrite: false;
		}

		ask farmerB {
			save [name, my_area, total_water_demand, cash] to: "../results/each_result.csv" type: "csv" rewrite: false;
		}

		ask farmerC {
			save [name, my_area, total_water_demand, cash] to: "../results/each_result.csv" type: "csv" rewrite: false;
		}

		do pause;
	}

}

species water { //agent parent that provides water to cropfields
	list<cropfield> my_fields; //cropfields inside the basin area
	float withdraw; //dayly limit to withdraw (Q95 of the day)
	date prosp_date; // date the helps to calc water availability on diferent stages of crop growth (used when choosing new crops)
	float f0; //duration of diferent stages of crop growth
	float pwithdraw; // water availability while choosing new crops
	float regional_demand; // demand inside basin

	reflex { // first cycle, defining as provider of water of cropfields inside
		if (cycle <= 0) {
			ask cropfield {
				if (self overlaps myself) {
					add self to: myself.my_fields;
					my_sourc <- myself;
				}

			}

		}

	}

	action prosp_water { //action used to provide water data while choosing new crops
	}

	aspect geom {
		draw shape color: #blue;
	}

}

species cor_rat parent: water { //agent child, sub-basin provides water to cropfields inside it

	float sp_flow; //variable that provides the amount of usable water (it is on liters/seconds/km²)
	float withdraw; //amount of water available in one day
	date prosp_date; // date used to calc water on diferent steps of crop growth (while choosing new crops)
	float f0; //amount of days of each crop gorwth step
	float pwithdraw; // water available on each step of crop growth 

	reflex recharge { // reflex that calc the amount of water available each day
		matrix data <- matrix(frato);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float(current_date.month)) {
				sp_flow <- data[1, i];
			}
		}
		withdraw <- (self.shape.area) * sp_flow * 0.0864; //conveting l/s/km² to l/day/m² TODO less water *0,7
	}

	reflex { //calc the water demand inside the basin
		regional_demand <- 0;
		ask my_fields {
			myself.regional_demand <- myself.regional_demand + self.wdem;
		}

	}

	action prosp_water { //action to mesure water availabily while choosing new crops
		date prosp_date <- current_date;
		matrix data <- matrix(frato);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float((prosp_date add_days f0).month)) {
				sp_flow <- data[1, i];
			}

			pwithdraw <- (self.shape.area) * sp_flow * 0.0864; // TODO less water *0,7
		}

	}

	aspect geom {
		draw shape color: #blue border: #black;
	}

}

species sam_north parent: water { //agent child, sub-basin provides water to cropfields inside it
//same specification of species cor_rat
	float sp_flow;
	float withdraw;
	date prosp_date;
	float f0;
	float pwithdraw;

	reflex recharge {
		matrix data <- matrix(fsamn);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float(current_date.month)) {
				sp_flow <- data[1, i];
			}

		}

		withdraw <- (self.shape.area) * sp_flow * 0.0864;
	}

	reflex {
		regional_demand <- 0;
		ask my_fields {
			myself.regional_demand <- myself.regional_demand + self.wdem;
		}

	}

	action prosp_water {
		date prosp_date <- current_date;
		matrix data <- matrix(fsamn);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float((prosp_date add_days f0).month)) {
				sp_flow <- data[1, i];
			}

			pwithdraw <- (self.shape.area) * sp_flow * 0.0864;
		}

	}

	aspect geom {
		draw shape color: #blue border: #black;
	}

}

species sam_south parent: water { //agent child, sub-basin provides water to cropfields inside it
//same specification of species cor_rat
	float sp_flow;
	float withdraw;
	date prosp_date;
	float f0;
	float pwithdraw;

	reflex recharge {
		matrix data <- matrix(fsams);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float(current_date.month)) {
				sp_flow <- data[1, i];
			}

		}

		withdraw <- (self.shape.area) * sp_flow * 0.0864;
	}

	reflex {
		regional_demand <- 0;
		ask my_fields {
			myself.regional_demand <- myself.regional_demand + self.wdem;
		}

	}

	action prosp_water {
		date prosp_date <- current_date;
		matrix data <- matrix(fsams);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float((prosp_date add_days f0).month)) {
				sp_flow <- data[1, i];
			}

		}

		pwithdraw <- (self.shape.area) * sp_flow * 0.0864;
	}

	aspect geom {
		draw shape color: #blue border: #black;
	}

}

species watershed { // species added just to ilustrate the area
	aspect geom {
		draw shape color: #green;
	}

}

species weather { //agent that provides weather data to cropfield, weather data is used to calc water demand and the amount of rain 
	float real_total_rain; //total rain of a month
	float real_temp_min; //minimun temperature of the month
	float real_temp_max; // maximun temperature of the month
	float rain; //rain retrived from total rein of the month
	float ETo; //base evapotranspiration (used to calc crop water demand)
	float HC <- 0.00084; //local parameter to calc evapotransp
	float Ra <- 0.5; //local parameter to calc evapotransp
	float HE <- 0.96; //local parameter to calc evapotransp
	float prosp_ETo; // prosp variables are used when the cropfield is prospectin new crops to produce
	float prosp_rain;
	float prosp_temp_min;
	float prosp_temp_max;
	float f0; //time of development of the crop in prospect steps
	date prosp_date; // each step of crop growth it has diferent water demand, this value help to prospect demands on diferent growth steps

	action calc_max_rain { //every new month the weather data is recalc to provide new data to cropfields
		matrix data <- matrix(weatherdata);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float(count_year) and data[1, i] = float(current_date.month)) {
				real_total_rain <- float(data[5, i]);//TODO less rain *0,7
				real_temp_min <- data[2, i];
				real_temp_max <- data[4, i];
			}

		}

		ETo <- 0.408 * HC * Ra * (real_temp_max - real_temp_min) ^ (HE) * (((real_temp_max + real_temp_min) / 2) + 17.38);
	}

	reflex { //reflex that calls "calc_max_rain" at begining of the month
		if (float(current_date.day) = 1) {
			do calc_max_rain;
		}

	}

	action prosp_weather { //action called from cropfield, it provides weather data from last year to help them choose new crop
		prosp_date <- current_date;
		matrix data <- matrix(weatherdata);
		loop i from: 0 to: data.rows - 1 {
			if (data[0, i] = float(last_year) and data[1, i] = float((prosp_date add_days f0).month)) { 
				prosp_rain <- float(data[5, i]);//TODO less rain *0,7
				prosp_temp_min <- data[2, i];
				prosp_temp_max <- data[4, i];
				prosp_ETo <- 0.408 * HC * Ra * (prosp_temp_max - prosp_temp_min) ^ (HE) * (((prosp_temp_max + prosp_temp_min) / 2) + 17.38);
			}

		}

	}

	reflex { // reflex that distributes that total rain of the month throughout the days
		bool rain_event;
		rain_event <- (flip(0.5) ? true : false);
		if (rain_event = true) {
			rain <- rnd(real_total_rain);
			real_total_rain <- real_total_rain - rain;
		} else {
			rain <- 0;
		}

	}

}

species visual_river { // species added just to ilustrate the river
	aspect geom {
		draw shape color: #blue;
	}

}

species calendar { // agent that provides data about the crops (when to plant, costs and produciton of each)
	list<int> dispcrops; //list of available crops
	int crop; //number of the crop
	float croptime; //total time to harvest 
	float deprewsense; //sensibility of the crop about lack of water (% of production lost due to lack of water)
	float crop_prod; //produced Kg per m^2
	float prod_cost; //producition costs
	float f1; //initial timer of the crop
	float f2; // development of the crop
	float f3; // mature crop
	float f4; // finishing crop
	float kc1; // crop coefficient on f1
	float kc2; // crop coefficient on f2
	float kc3; // crop coefficient on f3
	float kc4; // crop coefficient on f4
	file t_crop; //texture of the crop

	reflex availablecrop { //reflex that provides the crops available to plant each month
		if (current_date.month = 1) {
			dispcrops <- [0, 3, 7];
		} else if (current_date.month = 2) {
			dispcrops <- [0, 2, 3, 4, 6, 7, 8];
		} else if (current_date.month = 3) {
			dispcrops <- [0, 2, 4, 5, 6, 8];
		} else if (current_date.month = 4) {
			dispcrops <- [0, 4, 8, 5, 9];
		} else if (current_date.month = 5) {
			dispcrops <- [0, 4, 8, 9];
		} else if (current_date.month = 6) {
			dispcrops <- [0, 4, 8, 9];
		} else if (current_date.month = 7) {
			dispcrops <- [0];
		} else if (current_date.month = 8) {
			dispcrops <- [0];
		} else if (current_date.month = 9) {
			dispcrops <- [0, 2];
		} else if (current_date.month = 10) {
			dispcrops <- [0, 1, 2, 3, 4];
		} else if (current_date.month = 11) {
			dispcrops <- [0, 1, 2, 3, 4];
		} else if (current_date.month = 12) {
			dispcrops <- [0, 1, 3, 4];
		} }

	action dayly_demand { //action that provides crops data to cropfields
		if (crop = 1) {
			kc1 <- 0.6;
			kc2 <- 0.75;
			kc3 <- 1.08;
			kc4 <- 0.75;
			crop_prod <- 0.42;
			prod_cost <- 0.4384;
			croptime <- 120;
			deprewsense <- 26; // percentage of production lost due to be all crop with no water
			f1 <- 15;
			f2 <- 80;
			f3 <- 20;
			f4 <- 5;
			t_crop <- file("../includes/t_soy.jpg");
		} else if (crop = 2) {
			kc1 <- 0.6;
			kc2 <- 0.78;
			kc3 <- 1.13;
			kc4 <- 0.88;
			crop_prod <- 1.26;
			prod_cost <- 0.5119;
			croptime <- 120;
			deprewsense <- 26;
			f1 <- 20;
			f2 <- 34;
			f3 <- 40;
			f4 <- 26;
			t_crop <- file("../includes/t_corn.jpg");
		} else if (crop = 3) {
			kc1 <- 0.6;
			kc2 <- 0.75;
			kc3 <- 1.15;
			kc4 <- 0.85;
			crop_prod <- 0.45;
			prod_cost <- 1.0564;
			croptime <- 150;
			deprewsense <- 62;
			f1 <- 15;
			f2 <- 70;
			f3 <- 55;
			f4 <- 10;
			t_crop <- file("../includes/t_cotton.jpg");
		} else if (crop = 4) {
			kc1 <- 0.6;
			kc2 <- 0.75;
			kc3 <- 1.13;
			kc4 <- 0.7;
			crop_prod <- 0.3;
			prod_cost <- 0.4090;
			croptime <- 90;
			deprewsense <- 20;
			f1 <- 15;
			f2 <- 45;
			f3 <- 20;
			f4 <- 10;
			t_crop <- file("../includes/t_bean.jpg");
		} else if (crop = 5) {
			kc1 <- 0.6;
			kc2 <- 0.75;
			kc3 <- 1.13;
			kc4 <- 0.8;
			crop_prod <- 6;
			prod_cost <- 1.1309;
			croptime <- 130;
			deprewsense <- 50;
			f1 <- 25;
			f2 <- 30;
			f3 <- 45;
			f4 <- 30;
			t_crop <- file("../includes/t_potato.jpg");
		} else if (crop = 6) {
			kc1 <- 0.6;
			kc2 <- 0.85;
			kc3 <- 1.05;
			kc4 <- 0.75;
			crop_prod <- 1.6;
			prod_cost <- 5.7652;
			croptime <- 150;
			deprewsense <- 75;
			f1 <- 24;
			f2 <- 63;
			f3 <- 53;
			f4 <- 10;
			t_crop <- file("../includes/t_garlic.jpg");
		} else if (crop = 7) {
			kc1 <- 0.6;
			kc2 <- 0.75;
			kc3 <- 1.03;
			kc4 <- 0.88;
			crop_prod <- 6.0;
			prod_cost <- 1.4655;
			croptime <- 130;
			deprewsense <- 17;
			f1 <- 20;
			f2 <- 30;
			f3 <- 40;
			f4 <- 40;
			t_crop <- file("../includes/t_onion.jpg");
		} else if (crop = 8) {
			kc1 <- 0.6;
			kc2 <- 0.63;
			kc3 <- 0.85;
			kc4 <- 0.63;
			crop_prod <- 9.5;
			prod_cost <- 2.2045;
			croptime <- 125;
			deprewsense <- 88;
			f1 <- 28;
			f2 <- 37;
			f3 <- 37;
			f4 <- 23;
			t_crop <- file("../includes/t_tomato.jpg");
		} else if (crop = 9) {
			kc1 <- 0.6;
			kc2 <- 0.63;
			kc3 <- 1.13;
			kc4 <- 0.7;
			crop_prod <- 0.6;
			prod_cost <- 0.3505;
			croptime <- 110;
			deprewsense <- 53;
			f1 <- 20;
			f2 <- 50;
			f3 <- 30;
			f4 <- 10;
			t_crop <- file("../includes/t_wheat.jpg");
		} else if (crop = 0) {
			kc1 <- 0;
			kc2 <- 0;
			kc3 <- 0;
			kc4 <- 0;
			crop_prod <- 0;
			prod_cost <- 0.01;
			croptime <- 1;
			deprewsense <- 0;
			f1 <- 1;
			f2 <- 1;
			f3 <- 1;
			f4 <- 1;
			t_crop <- file("../includes/t_soil.jpg");
		} } }

species market { // agent that provides sell values of each crop each month
	float sellv; //sell value of the crop (price for each kg produced)
	float sold; 
	float deprew; //% of loss due to lack of water
	float my_area; // area of the cropfield
	float crop_prod; // production of the crop (kg produced on each m²)
	int farmercrop; // the type of crop

	action update_sellv { // action called to provide the selling price of a crop
		if (farmercrop = 0) {
			sellv <- 0;
		} else if (farmercrop != nil) {
			matrix data <- matrix(prices);
			loop i from: 0 to: data.rows - 1 {
				if (data[0, i] = count_year and data[1, i] = float(current_date.month)) {
					sellv <- float(data[(farmercrop + 1), i]);
				}

			}

		}

	}

	action buy_crop { //action called at the end of crop cycle, it gives cash depending on the amount produced and %loss due to lack of water
		if (deprew != 0) {
			sold <- (my_area * sellv * (crop_prod * (1 - ((deprew) / 100))));
		} else if (deprew = 0) {
			sold <- sellv * crop_prod * my_area;
		}

		if (farmercrop = 1) { //section to keep track of the amount produced (kg) and total area used to produce (m²)
			totalcrop1 <- totalcrop1 + my_area;
			totalkgcrop1 <- totalkgcrop1 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 2) {
			totalcrop2 <- totalcrop2 + my_area;
			totalkgcrop2 <- totalkgcrop2 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 3) {
			totalcrop3 <- totalcrop3 + my_area;
			totalkgcrop3 <- totalkgcrop3 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 4) {
			totalcrop4 <- totalcrop4 + my_area;
			totalkgcrop4 <- totalkgcrop4 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 5) {
			totalcrop5 <- totalcrop5 + my_area;
			totalkgcrop5 <- totalkgcrop5 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 6) {
			totalcrop6 <- totalcrop6 + my_area;
			totalkgcrop6 <- totalkgcrop6 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 7) {
			totalcrop7 <- totalcrop7 + my_area;
			totalkgcrop7 <- totalkgcrop7 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 8) {
			totalcrop8 <- totalcrop8 + my_area;
			totalkgcrop8 <- totalkgcrop8 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} else if (farmercrop = 9) {
			totalcrop9 <- totalcrop9 + my_area;
			totalkgcrop9 <- totalkgcrop9 + my_area * (crop_prod * (1 - ((deprew) / 100)));
		} } }

species farmer { //agent parent of farmers they own cropfields, pay for upkeepcosts, irrigation costs,receive money when sell crops and choose new type of crop produced
	float cash; // money
	float farmercrop; //type of crop (0-nocrop, 1-soybean, 2-cotton, 3-corn, 4-beans, 5-potato, 6-garlic, 7-onion, 8-tomato and 9-wheat)
	float last_crop; //last type of crop produced, to avoid producing the same crop 2 times in a roll, its forbidden due phytosanitary issues
	float timer; //timer to check count crop time
	float day_wcost; //cost of water on a day
	float day_wdem; //demand of water (number of cells * type of crop)
	float croptime; // days to havest
	list<cropfield> my_cells; //list of cells owned by farmer
	float totaldeprew; //total depreciation of crops due to lack of water
	int nb_cells; //number of cells(cropfields) the farmer owns
	float my_area; //sum of areas of cells owned
	float expect_revenue; // added to check the expected revenue
	float total_upkeepcost; //total cash spend on upkeep
	float upkeepcost; //upkeep cost to calc expected profit
	int prosp_farmercrop; //prospect crop type
	float prospect_upcost; //prospect upkeep cost
	float prospect_sellvalue; //prospect sell value
	float prospect_wcost; // prospect irrigation cost
	float sellv_index; //weight of sell value on choosing new crop
	float upcost_index; //weight of upkeep cost on choosing new crop
	float wcost_index;//weight of irrigation on choosing new crop
	float prosp_crop;//crop prospected
	float prosped_crop;//crop choosen
	list<int> dispcrops;//crops available to prospect
	float ks <- 1; //variable to calc ETo, variable about soil condiction
	//variables to save
	float wdem; // water demand
	float total_water_demand; //total water demand


	reflex {
		if (cycle <= 0) {//choosing the cropfields, it picks the closest fiels with no owner
			list<cropfield> nofarmer <- cropfield where (each.my_farmer = nil);
			my_cells <- nofarmer closest_to (self, nb_cells);
			ask my_cells {
				my_farmer <- myself;
			}

			my_area <- sum(my_cells collect each.shape.area);
			cash <- my_area * 0.1; //initial cash
			do update_farmercrop; //choose new crop
			ask my_cells {
				do update_dayly_demand;
			}

		}

	}

	reflex expected_profit { //indicator
		expect_revenue <- sum(my_cells collect each.expect_value);
		upkeepcost <- sum(my_cells collect each.upcost);
		day_wcost <- sum(my_cells collect each.wcost);
		day_wdem <- sum(my_cells collect each.wdem);
		total_upkeepcost <- upkeepcost * croptime;
	}

	reflex update_payment {// daily costs
		ask my_cells {
			do costs;
			myself.cash <- myself.cash - self.wcost;
			myself.cash <- myself.cash - self.upcost;
			myself.totaldeprew <- myself.totaldeprew + self.deprew;
		}

	}

	reflex timer_crop { //timer to harvest
		if (farmercrop = farmercrop) {
			timer <- timer + 1;
			if (timer = croptime) {
				do sell_crop;
				farmercrop <- nil;
				do update_farmercrop;
				timer <- 0;
			}

		}

	}

	action update_farmercrop { //choose new crop
		if (farmercrop = nil or farmercrop = 0) {
			ask calendar { //updating the list of crops that can be planted this month
				myself.dispcrops <- self.dispcrops;
			}

			prosped_crop <- 0;
			loop i over: dispcrops { 
				prosp_farmercrop <- i;
				if (prosp_farmercrop != last_crop) {
					ask one_of(my_cells) { //make the one of the cropfields check for all the availble crop data, which one fits better on the criteria is choosen as new crop
						do prospect;
						myself.prospect_upcost <- self.prospect_upcost;
						myself.prospect_sellvalue <- self.prospect_sellvalue;
						myself.prospect_wcost <- self.prospect_wcost;
						myself.prosp_crop <- (myself.prospect_sellvalue * myself.sellv_index) - (myself.prospect_upcost * myself.upcost_index) - (myself.prospect_wcost * myself.wcost_index);
						if (myself.prosp_crop > myself.prosped_crop) {
							myself.prosped_crop <- myself.prosp_crop;
							myself.farmercrop <- i;
						}

					}

				}

			}

			ask calendar { //plant the new crop
				self.crop <- myself.farmercrop;
				do dayly_demand;
				myself.croptime <- self.croptime;
			}

			ask my_cells {
				do update_dayly_demand;
			}

		}

	}

	action sell_crop {//when the crop cycle is over, the production is sold
		if (farmercrop != 0) {
			last_crop <- farmercrop;
		}

		ask my_cells {
			do sell_crop;
			myself.cash <- myself.cash + self.selling;
			self.selling <- 0;
		}

		totaldeprew <- 0;
	}

	aspect default {
	}

	aspect default_cells {
	}

	reflex saving_atributes { //indicator
		ask my_cells {
			myself.total_water_demand <- myself.total_water_demand + self.wdem;
		}

	}

}

species farmerC parent: farmer {
	float sellv_index <- rnd(0.5, 0.8); //weight of sell value on choosing new crop
	float upcost_index <- rnd(1.2, 1.5); // weight of upkeep cost on choosing new crop
	float wcost_index <- rnd(1.2, 1.5); // weight of irrigation cost (water cost) on choosing new crop
	int nb_cells <- 10; // number of cells owned
}

species farmerB parent: farmer {
	//same specification of farmerC
	float sellv_index <- rnd(1.2, 1.5);
	float upcost_index <- rnd(0.5, 0.8);
	float wcost_index <- rnd(0.5, 0.8);
	int nb_cells <- 30;
}

species farmerA parent: farmer {
	//same specification of farmer C
	float sellv_index <- rnd(0.8, 1.2);
	float upcost_index <- rnd(0.8, 1.2);
	float wcost_index <- rnd(0.8, 1.2);
	int nb_cells <- 50;
}

species cropfield {
	farmer my_farmer; //owner of cropfield
	water my_sourc; //source of water (basin that provides water)
	file t_crop; // texture shown (change deppending on the type of crop)
	float deprew; //price depreciation due to lack of water
	float deprewsense; //crop sensibility to dry days
	float croptime; // days to havest
	float wdem; //demand of water (area * type of crop)
	float wcost; //daily cost of water
	float upcost; //daily upkeep cost
	float upkeep_cost; //upkeep cost an indicator
	float selling; // variable used on sell action
	float missingw; // missing quantity of water to fullfill the needs
	float cropmarker; //add just to mark the crop number on the cropfield and make the couting easy
	float crop_prod; // production of the crop
	float actual_loss; //percentual of production lost by day without water
	float prod_cost; //cost to mantein 1m^2 for the duration of the crop	
	float field_area; //area of the field (added to check numbers of the model)
	float expect_value; //expected value of the production
	float dry_area; //to keep track of dry area (area without enought water to irrigate)
	float f1; //initial timer of the crop
	float f2; // development of the crop
	float f3; // mature crop
	float f4; // finishing crop
	float kc1; // coeficiente de cultura no f1
	float kc2; // coeficiente de cultura no f2
	float kc3; // coeficiente de cultura no f3
	float kc4; // coeficiente de cultura no f4
	float ETo; //base evapotransp
	float kc; //crop coeficien
	float ks <- 1; // soil coeficient
	float ftime; //time of development phase f1,f2,f3,f4
	float prosp_actual_loss; //"prop_" are variables used on choosing new crops
	float prospect_upcost;
	float prospect_sellvalue;
	float prospect_wcost;
	float prosp_wdem;
	float prosp_wdem1; //wdem of the probable crop at f1
	float prosp_wdem2;
	float prosp_wdem3;
	float prosp_wdem4;
	float prosp_rain1; //parcial rain on prospect time
	float prosp_rain2;
	float prosp_rain3;
	float prosp_rain4;
	float mean_prosp_rain;
	float prosp_flow1; // parcial flow on prospect time
	float prosp_flow2;
	float prosp_flow3;
	float prosp_flow4;
	float prosp_meanflow;
	float prosp_withd1; // parcial withdraw on prospect time
	float prosp_withd2;
	float prosp_withd3;
	float prosp_withd4;
	float prosp_meanwithd;

	init {
		t_crop <- file("../includes/t_soil.jpg");
		field_area <- self.shape.area; //(indicator)
	}

	aspect geom {
		draw shape texture: [t_crop];
	}

	action update_dayly_demand { //update the data of the crop
		ask calendar { //gather data of the crop
			self.crop <- myself.my_farmer.farmercrop;
			do dayly_demand;
			myself.crop_prod <- self.crop_prod;
			myself.prod_cost <- self.prod_cost;
			myself.croptime <- self.croptime;
			myself.deprewsense <- self.deprewsense;
			myself.t_crop <- self.t_crop;
			myself.f1 <- self.f1;
			myself.f2 <- self.f2;
			myself.f3 <- self.f3;
			myself.f4 <- self.f4;
			myself.kc1 <- self.kc1;
			myself.kc2 <- self.kc2;
			myself.kc3 <- self.kc3;
			myself.kc4 <- self.kc4;
		}

		ask weather {
			myself.ETo <- self.ETo;
		}

		actual_loss <- deprewsense / croptime;
	}

	reflex { //provide the water demand of the crop, depending on the growth level
		if (my_farmer != nil) {
			ask weather {
				myself.ETo <- self.ETo;
			}

			cropmarker <- my_farmer.farmercrop;
			if (my_farmer.timer <= f1) {
				kc <- kc1;
				ftime <- f1;
			} else if (my_farmer.timer > f1 and my_farmer.timer <= f1 + f2) {
				kc <- kc2;
				ftime <- f2;
			} else if (my_farmer.timer > f1 + f2 and my_farmer.timer <= f1 + f2 + f3) {
				kc <- kc3;
				ftime <- f3;
			} else if (my_farmer.timer > f1 + f2 + f3 and my_farmer.timer <= f1 + f2 + f3 + f4) {
				kc <- kc4;
				ftime <- f4;
			}

			wdem <- (ETo * kc * ks * field_area);
		} }

	action costs { //calc the daily costs of the crop
		ask weather { //check for rain to diminish the water demand
			myself.wdem <- myself.wdem - (self.rain * myself.field_area);
			if (myself.wdem < 0) {
				myself.wdem <- 0;
			}

		}

		ask my_sourc { //take water from the source
			if (self.withdraw > myself.wdem) {
				self.withdraw <- self.withdraw - myself.wdem;
				myself.wcost <- myself.wdem * ppcost; //daily irrigation cost
			} else if (self.withdraw < myself.wdem) { //if there is not enought water, a depreciation will be added (which will reduce the amount of production)
				myself.missingw <- myself.wdem - self.withdraw;
				if (self.withdraw < 0) {
					self.withdraw <- 0;
				}

				myself.deprew <- myself.deprew + (myself.actual_loss * (myself.missingw / myself.wdem));
				myself.dry_area <- myself.dry_area + (myself.shape.area * (myself.missingw / myself.wdem));
				myself.wcost <- (myself.missingw - myself.wdem) * ppcost;
				if (myself.wcost < 0) {
					myself.wcost <- myself.wcost * -1;
				}

			}

			totalflowdemand <- totalflowdemand + myself.wdem; //indicator
		}

		upkeep_cost <- (prod_cost / croptime); 
		upcost <- upkeep_cost * field_area;//daily upkeep cost
	}

	reflex up_expect { // indicator
		if (my_farmer != nil) {
			ask market {
				self.farmercrop <- myself.cropmarker;
				self.deprew <- myself.deprew;
				self.my_area <- myself.shape.area;
				self.crop_prod <- myself.crop_prod;
				do update_sellv;
				myself.expect_value <- myself.field_area * myself.crop_prod * self.sellv;
			}

		}

	}

	action prospect { //action that picks data to choose new crop
		ask calendar { //taker the data of the prospected type of crop
			self.crop <- myself.my_farmer.prosp_farmercrop;
			do dayly_demand;
			myself.prod_cost <- self.prod_cost;
			myself.croptime <- self.croptime;
			myself.crop_prod <- self.crop_prod;
			myself.deprewsense <- self.deprewsense;
			myself.f1 <- self.f1;
			myself.f2 <- self.f2;
			myself.f3 <- self.f3;
			myself.f4 <- self.f4;
			myself.kc1 <- self.kc1;
			myself.kc2 <- self.kc2;
			myself.kc3 <- self.kc3;
			myself.kc4 <- self.kc4;
		}

		prosp_actual_loss <- deprewsense;
		prospect_upcost <- prod_cost / croptime;
		ask weather { //based on last year information, check if there would be rain for the prospected crop
			if (myself.my_farmer != nil) {
				self.f0 <- myself.f1;
				myself.kc <- myself.kc1;
				do prosp_weather;
				myself.prosp_wdem1 <- (self.prosp_ETo * myself.kc * myself.ks * myself.f1); 
				loop i from: 0 to: myself.f1 {
					bool i;
					float prosp_rain_ev;
					i <- (flip(0.5) ? true : false);
					if (i = true) {
						prosp_rain_ev <- rnd(myself.prosp_rain1);
						if (prosp_rain_ev >= (myself.prosp_wdem1 / myself.f1)) {
							myself.prosp_wdem1 <- myself.prosp_wdem1 - (myself.prosp_wdem1 / myself.f1);
						} else {
							myself.prosp_wdem1 <- myself.prosp_wdem1 - prosp_rain_ev;
						}

						myself.prosp_rain1 <- myself.prosp_rain1 - prosp_rain_ev;
					} else {
						prosp_rain_ev <- 0.0;
						myself.prosp_wdem1 <- myself.prosp_wdem1 - prosp_rain_ev;
					}

				}

				self.f0 <- myself.f1 + myself.f2;
				myself.kc <- myself.kc2;
				do prosp_weather;
				myself.prosp_wdem2 <- (self.prosp_ETo * myself.kc * myself.ks * myself.f2); 
				loop i from: 0 to: myself.f2 {
					bool i;
					float prosp_rain_ev;
					i <- (flip(0.5) ? true : false);
					if (i = true) {
						prosp_rain_ev <- rnd(myself.prosp_rain2);
						if (prosp_rain_ev >= (myself.prosp_wdem2 / myself.f2)) {
							myself.prosp_wdem2 <- myself.prosp_wdem2 - (myself.prosp_wdem2 / myself.f2);
						} else {
							myself.prosp_wdem2 <- myself.prosp_wdem2 - prosp_rain_ev;
						}

						myself.prosp_rain2 <- myself.prosp_rain2 - prosp_rain_ev;
					} else {
						prosp_rain_ev <- 0.0;
						myself.prosp_wdem2 <- myself.prosp_wdem2 - prosp_rain_ev;
					}

				}

				self.f0 <- myself.f1 + myself.f2 + myself.f3;
				myself.kc <- myself.kc3;
				do prosp_weather;
				myself.prosp_wdem3 <- (self.prosp_ETo * myself.kc * myself.ks * myself.f3); 	
				loop i from: 0 to: myself.f3 {
					bool i;
					float prosp_rain_ev;
					i <- (flip(0.5) ? true : false);
					if (i = true) {
						prosp_rain_ev <- rnd(myself.prosp_rain3);
						if (prosp_rain_ev >= (myself.prosp_wdem3 / myself.f3)) {
							myself.prosp_wdem3 <- myself.prosp_wdem3 - (myself.prosp_wdem3 / myself.f3);
						} else {
							myself.prosp_wdem3 <- myself.prosp_wdem3 - prosp_rain_ev;
						}

						myself.prosp_rain3 <- myself.prosp_rain3 - prosp_rain_ev;
					} else {
						prosp_rain_ev <- 0.0;
						myself.prosp_wdem3 <- myself.prosp_wdem3 - prosp_rain_ev;
					}

				}

				self.f0 <- myself.f1 + myself.f2 + myself.f3 + myself.f4;
				myself.kc <- myself.kc4;
				do prosp_weather;
				myself.prosp_wdem4 <- (self.prosp_ETo * myself.kc * myself.ks * myself.f4); 
				loop i from: 0 to: myself.f4 {
					bool i;
					float prosp_rain_ev;
					i <- (flip(0.5) ? true : false);
					if (i = true) {
						prosp_rain_ev <- rnd(myself.prosp_rain4);
						if (prosp_rain_ev >= (myself.prosp_wdem4 / myself.f4)) {
							myself.prosp_wdem4 <- myself.prosp_wdem4 - (myself.prosp_wdem4 / myself.f4);
						} else {
							myself.prosp_wdem4 <- myself.prosp_wdem4 - prosp_rain_ev;
						}

						myself.prosp_rain4 <- myself.prosp_rain4 - prosp_rain_ev;
					} else {
						prosp_rain_ev <- 0.0;
						myself.prosp_wdem4 <- myself.prosp_wdem4 - prosp_rain_ev;
					}

				}

				myself.prosp_wdem <- (myself.prosp_wdem1) + (myself.prosp_wdem2) + (myself.prosp_wdem3) + (myself.prosp_wdem4);
			}

			myself.prosp_rain1 <- 0;
			myself.prosp_rain2 <- 0;
			myself.prosp_rain3 <- 0;
			myself.prosp_rain4 <- 0;
		}

		ask my_sourc { //check if there is enought water for the prospected crop
			self.f0 <- myself.f1;
			do prosp_water;
			myself.prosp_withd1 <- self.pwithdraw * myself.f1;
			self.f0 <- myself.f1 + myself.f2;
			do prosp_water;
			myself.prosp_withd2 <- self.pwithdraw * myself.f2;
			self.f0 <- myself.f1 + myself.f2 + myself.f3;
			do prosp_water;
			myself.prosp_withd3 <- self.pwithdraw * myself.f3;
			self.f0 <- myself.f1 + myself.f2 + myself.f3 + myself.f4;
			do prosp_water;
			myself.prosp_withd4 <- self.pwithdraw * myself.f4;
			myself.prosp_meanwithd <- (myself.prosp_withd1 + myself.prosp_withd2 + myself.prosp_withd3 + myself.prosp_withd4) / myself.croptime;
			if (myself.prosp_meanwithd > (myself.prosp_wdem / myself.croptime)) {
				myself.prospect_wcost <- (myself.prosp_wdem / myself.croptime) * ppcost;
			} else if (myself.prosp_meanwithd < (myself.prosp_wdem / myself.croptime)) {
				myself.missingw <- myself.prosp_wdem - (myself.prosp_meanwithd * myself.croptime);
				if (myself.prosp_meanwithd > 0) {
					myself.prosp_meanwithd <- 0;
				}

				myself.deprew <- (myself.prosp_actual_loss * (myself.missingw / myself.prosp_wdem));
				myself.prospect_wcost <- myself.prosp_meanwithd * ppcost;
			}

		}

		ask market {//check the price of prospected crop
			self.farmercrop <- myself.my_farmer.prosp_farmercrop;
			self.deprew <- myself.deprew;
			self.my_area <- 1;
			self.crop_prod <- myself.crop_prod;
			do update_sellv;
			myself.prospect_sellvalue <- ((myself.crop_prod * self.sellv) / myself.croptime) * (1 - ((myself.deprew) / 100));
		}

		deprew <- 0;
	}

	action sell_crop { //sell the crop
		ask market {
			self.farmercrop <- myself.cropmarker;
			self.deprew <- myself.deprew;
			self.my_area <- myself.field_area;
			self.crop_prod <- myself.crop_prod;
			do update_sellv;
			do buy_crop;
			myself.selling <- self.sold;
		}

		deprew <- 0;
		crop_prod <- 0.01;
	} }

experiment 'Run Batch' type: batch repeat: 100 keep_seed: false until: (cycle = 3600) { //simulation without user interface, used to run several simulations and gather results
	reflex end_of_runs {
		int cpt <- 0;
		ask simulations {
			save [cpt, self.total_av_water, self.total_dem_water, self.total_cash,self.totalcrop1,self.totalcrop2,self.totalcrop3,self.totalcrop4,self.totalcrop5,self.totalcrop6,self.totalcrop7,self.totalcrop8,self.totalcrop9] to: "../results/scenario_total_result.csv" type: "csv" rewrite: false;
			ask farmerA {
				save [cpt, self.name,self.sellv_index,self.upcost_index,self.wcost_index, self.my_area, self.total_water_demand, self.cash] to: "../results/scenario_each_result.csv" type: "csv" rewrite: false;
			}

			ask farmerB {
				save [cpt, self.name,self.sellv_index,self.upcost_index,self.wcost_index, self.my_area, self.total_water_demand, self.cash] to: "../results/scenario_each_result.csv" type: "csv" rewrite: false;
			}

			ask farmerC {
				save [cpt, self.name,self.sellv_index,self.upcost_index,self.wcost_index, self.my_area, self.total_water_demand, self.cash] to: "../results/scenario_each_result.csv" type: "csv" rewrite: false;
			}

			cpt <- cpt + 1;
		}

	}

}

experiment myExp type: gui { //simulation with user interface
	output {
		display MyDisplay type: opengl { //display showing the map
			species watershed aspect: geom transparency: 0.05;
			species cor_rat aspect: geom transparency: 0.95;
			species sam_north aspect: geom transparency: 0.90;
			species sam_south aspect: geom transparency: 0.85;
			species cropfield aspect: geom;
			species farmerA aspect: default;
			species farmerA aspect: default_cells;
			species farmerB aspect: default;
			species farmerB aspect: default_cells;
			species farmerC aspect: default;
			species farmerC aspect: default_cells;
			species water aspect: geom;
			species visual_river aspect: geom;
		}

		display Crops_data { //graphs with crop data
			chart "Total Crop Area (m²)" type: histogram background: rgb("white") size: {1, 0.33} position: {0, 0} {
				data "Crop 1" value: totalcrop1 color: #brown;
				data "Crop 2" value: totalcrop2 color: #gray;
				data "Crop 3" value: totalcrop3 color: #yellow;
				data "Crop 4" value: totalcrop4 color: #crimson;
				data "Crop 5" value: totalcrop5 color: #crimson;
				data "Crop 6" value: totalcrop6 color: #crimson;
				data "Crop 7" value: totalcrop7 color: #crimson;
				data "Crop 8" value: totalcrop8 color: #crimson;
				data "Crop 9" value: totalcrop9 color: #crimson;
			}

			chart "Crop Now (un)" type: histogram background: rgb("white") size: {1, 0.33} position: {0, 0.33} {
				data "Crop 1" value: cropfield count (each.cropmarker = 1) color: #brown;
				data "Crop 2" value: cropfield count (each.cropmarker = 2) color: #gray;
				data "Crop 3" value: cropfield count (each.cropmarker = 3) color: #yellow;
				data "Crop 4" value: cropfield count (each.cropmarker = 4) color: #crimson;
				data "Crop 5" value: cropfield count (each.cropmarker = 5) color: #crimson;
				data "Crop 6" value: cropfield count (each.cropmarker = 6) color: #crimson;
				data "Crop 7" value: cropfield count (each.cropmarker = 7) color: #crimson;
				data "Crop 8" value: cropfield count (each.cropmarker = 8) color: #crimson;
				data "Crop 9" value: cropfield count (each.cropmarker = 9) color: #crimson;
			}

			chart "Total Crop production (kg)" type: histogram background: rgb("white") size: {1, 0.33} position: {0, 0.66} {
				data "Crop 1" value: totalkgcrop1 color: #brown;
				data "Crop 2" value: totalkgcrop2 color: #gray;
				data "Crop 3" value: totalkgcrop3 color: #yellow;
				data "Crop 4" value: totalkgcrop4 color: #crimson;
				data "Crop 5" value: totalkgcrop5 color: #crimson;
				data "Crop 6" value: totalkgcrop6 color: #crimson;
				data "Crop 7" value: totalkgcrop7 color: #crimson;
				data "Crop 8" value: totalkgcrop8 color: #crimson;
				data "Crop 9" value: totalkgcrop9 color: #crimson;
			}

		}

		display Regional_Demands { //graphs with local water data
			chart "Available water Vs Demand Vs Granted" type: histogram background: #white size: {1, 0.33} position: {0, 0} {
				data "Rato" value: cor_rat collect each.withdraw color: #blue;
				data "Rato_demand" value: cor_rat collect each.regional_demand color: #lightblue;
			}

			chart "Available water Vs Demand Vs Granted" type: histogram background: #white size: {1, 0.33} position: {0, 0.33} {
				data "Sam_North" value: sam_north collect each.withdraw color: #blue;
				data "Sam_North_demand" value: sam_north collect each.regional_demand color: #lightblue;
			}

			chart "Available water Vs Demand Vs Granted" type: histogram background: #white size: {1, 0.33} position: {0, 0.66} {
				data "Sam_South" value: sam_south collect each.withdraw color: #blue;
				data "Sam_South_demand" value: sam_south collect each.regional_demand color: #lightblue;
			}

		}

		display Resourses { //graphs with global water data
			chart "Total Resource Demand" type: series size: {1, 0.5} position: {0, 0} {
				data "Dayly Available Water" value: dayly_withdraw color: #blue;
				data "Dayly Water Demand" value: inst_wdem color: #lightblue;
			}

			chart "Available Resource" type: histogram size: {1, 0.5} position: {0, 0.5} {
				data "Dayly Available Water" value: dayly_withdraw color: #blue;
				data "Dayly Water Demand" value: inst_wdem color: #lightblue;
			}

		}

		display Wealth {//graphs with cash data
			chart "Total Wealth" type: series size: {1, 0.5} position: {0, 0} {
				data "Total Wealth" value: sum((farmerA, farmerB, farmerC) collect each.cash) color: #lightgreen;
			}

			chart "Distribution of farmers cash" type: pie style: exploded size: {1, 0.5} position: {0, 0.5} {
				datalist legend: farmerA collect each.name value: farmerA collect each.cash color: #lightgreen;
				datalist legend: farmerB collect each.name value: farmerB collect each.cash color: #lightgreen;
				datalist legend: farmerC collect each.name value: farmerC collect each.cash color: #lightgreen;
			}

		}
		//monitors to keep up with some numbers
		monitor "Dry_Area (m²)" value: global_dry_area;
		monitor "daily withdraw (mm)" value: dayly_withdraw;
		monitor "daily demand (mm)" value: inst_wdem;
		monitor "rain (mm)" value: basin_rain;
	}

}